package com.ruoyi.system.service.impl;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.ruoyi.common.constant.TokenConstants;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.system.domain.dto.request.WeChatCodeRequest;
import com.ruoyi.system.domain.dto.request.WeChatRequest;
import com.ruoyi.system.service.WeChatService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.Formatter;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 微信业务实现层
 *
 * @Author: wu
 * @DateTime: 2023/4/25 9:31
 */
@Service
public class WeChatServiceImpl implements WeChatService {

    @Value("${wechat.appid}")
    private String appid;

    @Value("${wechat.secret}")
    private String secret;

    @Resource
    private RedisCache redisCache;

    /**
     * sha摘要算法
     *
     * @param str 生成摘要的字符串
     * @return 摘要
     * @throws NoSuchAlgorithmException 算法不存在异常
     */
    private String SHA1(String str) throws NoSuchAlgorithmException {
        MessageDigest crypt = MessageDigest.getInstance("SHA-1");
        crypt.reset();
        crypt.update(str.getBytes());
        Formatter formatter = new Formatter();
        for (byte b : crypt.digest()) {
            formatter.format("%02x", b);
        }
        String result = formatter.toString();
        formatter.close();
        return result;
    }

    @Override
    public String findAccessToken() {
        String accessToken;
        accessToken = redisCache.getCacheObject(TokenConstants.ACCESS_TOKEN + appid);
        if (StrUtil.isNotEmpty(accessToken)) {
            return accessToken;
        }
        Map<String, Object> map = new HashMap<>(3);
        String grantType = "client_credential";
        map.put("grant_type", grantType);
        map.put("appid", appid);
        map.put("secret", secret);
        String body = HttpUtil.get("https://api.weixin.qq.com/cgi-bin/token", map);
        accessToken = (String) new JSONObject(body).get("access_token");
        // accessToken存入redis
        redisCache.setCacheObject(TokenConstants.ACCESS_TOKEN + appid, accessToken, 7200, TimeUnit.SECONDS);
        return accessToken;
    }

    @Override
    public String findJsapiTicket() {
        String ticket;
        ticket = redisCache.getCacheObject(TokenConstants.JsapiTicket + appid);
        if (StrUtil.isNotEmpty(ticket)) {
            return ticket;
        }
        String accessToken = findAccessToken();
        Map<String, Object> map = new HashMap<>(2);
        map.put("access_token", accessToken);
        String type = "jsapi";
        map.put("type", type);
        String body = HttpUtil.get("https://api.weixin.qq.com/cgi-bin/ticket/getticket", map);
        ticket = (String) new JSONObject(body).get("ticket");
        // accessToken存入redis
        redisCache.setCacheObject(TokenConstants.JsapiTicket + appid, ticket, 7200, TimeUnit.SECONDS);
        return ticket;
    }

    @Override
    public String findSignature(WeChatRequest req) {
        String signature;
        String jsapi_ticket = findJsapiTicket();
        String url = req.getUrl();
        String nonceStr = req.getNoncestr();
        String timestamp = req.getTimestamp();
        String[] arr = new String[]{"jsapi_ticket=" + jsapi_ticket, "noncestr=" + nonceStr, "timestamp=" + timestamp, "url=" + url};
        Arrays.sort(arr);
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < arr.length; i++) {
            sb.append(arr[i]);
            if (i != arr.length - 1) {
                sb.append("&");
            }
        }
        try {
            signature = SHA1(sb.toString());
        } catch (Exception e) {
            throw new ServiceException("未知异常,请联系管理员");
        }
        return signature;
    }

    /**
     * 获取微信手机号
     *
     * @param code 手机号获取凭证
     * @return 手机号
     */
    @Override
    public Map<String, Object> getUserPhoneNumber(String code) {
        String accessToken = this.findAccessToken();
        Map<String, Object> map = new HashMap<>();
        map.put("code", code);
        String jsonStr = JSONUtil.toJsonStr(map);
        String body = HttpUtil.post("https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token=" + accessToken, jsonStr);
        return new JSONObject(body);
    }

    /**
     * 获取微信二维码
     *
     * @param req 获取小程序码请求类
     * @return 微信二维码二进制
     */
    @Override
    public byte[] findWxCode(WeChatCodeRequest req) {
        String accessToken = this.findAccessToken();
        Map<String, Object> map = new HashMap<>();
        if (req.getWidth() != null) {
            map.put("width", req.getWidth());
        }
        map.put("path", req.getPath() + "?shanghuId=" + req.getShanghuId());
        String jsonStr = JSONUtil.toJsonStr(map);
        return HttpRequest.post("https://api.weixin.qq.com/wxa/getwxacode?access_token=" + accessToken)
                .body(jsonStr)
                .execute().bodyBytes();
    }
}
